/***************************************************************************
 *   Copyright (C) 2007  Antony Martin <martin.antony@yahoo.fr>            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU General Public License           *
 *   as published by the Free Software Foundation; either version 2        *
 *   of the License, or (at your option) any later version.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA          *
 ***************************************************************************/

/* Cree le : 14 janvier 2007
   derniere modification le 11/01/2008 */

#ifndef SCECTEXTURE_H
#define SCECTEXTURE_H

#include <stdarg.h>
#include <SCE/utils/SCEList.h>
#include <SCE/core/SCECImage.h>

#ifdef __cplusplus
extern "C"
{
#endif

/* ajoute le 03/10/2007 */
/* revise le 11/01/2008 */
typedef struct sce_ctexdata SCE_CTexData;
struct sce_ctexdata
{
    SCE_CImage *img;
    int canfree;       /* si 'img' appartient a l'utilisateur */
    int user;          /* si TOUTE la structure appartient a l'utilisateur */
    SCEenum target;
    int level;
    int w, h, d;
    SCEenum pxf;
    SCEenum fmt;
    SCEenum type;
    size_t data_size;
    int data_user;   /* si 'data' appartient a l'utilisateur */
    void *data;
    int comp;
};

/* revise le 07/10/2007 */
typedef struct sce_ctexture SCE_CTexture;
struct sce_ctexture
{
    SCEuint id;             /*! OpenGL identifier */
    SCEenum target;         /*! Target (type) of the texture */
    SCE_SList *data[6];     /*! Data of the texture */
    int have_data;          /*! True if have data */

    int use_mipmap, hw_mipmap;

    /* ajoute le 02/10/2007 */
    enum SCE_ETexType
    {
        SCE_TEXTYPE_1D, SCE_TEXTYPE_2D, SCE_TEXTYPE_3D, SCE_TEXTYPE_CUBE
    } type;
};


int SCE_CTextureInit (void);
void SCE_CTextureQuit (void);

/* defini la texture active */
void SCE_CBindTexture (SCE_CTexture*);

/* initialise une structure de donnees d'une texture */
void SCE_CInitTexData (SCE_CTexData*);
/* cree une donnee de texture */
SCE_CTexData* SCE_CCreateTexData (void);
/* cree et initialise une donnee de texture a partir d'une image (bindee) */
SCE_CTexData* SCE_CCreateTexDataFromImage (void);
/* supprime une donnee de texture */
void SCE_CDeleteTexData (void*);
/* duplique dans son integralite une donnee de texture */
SCE_CTexData* SCE_CDupTexData (SCE_CTexData*);

/* cree une texture */
SCE_CTexture* SCE_CCreateTexture (SCEenum);
/* supprime une texture */
void SCE_CDeleteTexture (SCE_CTexture*);
void SCE_CDeleteTexture_ (void);

/* retourne le nombre maximum d'unites de texture */
unsigned int SCE_CGetMaxTextureUnits (void);

/* installe un filtre a une texture */
void SCE_CSetTextureFilters (SCE_CTexture*, SCEint);
void SCE_CSetTextureFilters_ (SCEint);

/* envoie un parametre specifie par l'utilisateur */
void SCE_CSetTextureParam (SCE_CTexture*, SCEenum, int);
void SCE_CSetTextureParam_ (SCEenum, int);
void SCE_CSetTextureParamf (SCE_CTexture*, SCEenum, float);
void SCE_CSetTextureParamf_ (SCEenum, float);

/* generation des coordonnees de texture */
void SCE_CSetTextureGen (SCE_CTexture*, SCEenum, SCEenum, int);
void SCE_CSetTextureGen_ (SCEenum, SCEenum, int);
void SCE_CSetTextureGenf (SCE_CTexture*, SCEenum, SCEenum, float);
void SCE_CSetTextureGenf_ (SCEenum, SCEenum, float);
void SCE_CSetTextureGenfv (SCE_CTexture*, SCEenum, SCEenum, float*);
void SCE_CSetTextureGenfv_ (SCEenum, SCEenum, float*);

/* renvoie le target de la texture */
SCEenum SCE_CGetTextureTarget (SCE_CTexture*);
SCEenum SCE_CGetTextureTarget_ (void);

/* renvoie la donnee de texture correspondante (target & mipmap level) */
SCE_CTexData* SCE_CGetTextureTexData (SCE_CTexture*, int, int);
SCE_CTexData* SCE_CGetTextureTexData_ (int, int);

/* renvoie le nombre de niveaux de mipmaps deja presents dans les donnees */
int SCE_CGetTextureNumMipmaps (SCE_CTexture*, int);
int SCE_CGetTextureNumMipmaps_ (int);

/* renvoie la dimension valide (en focntion du materiel) la plus proche
   de celle recue (plus proche superieur ou inferieur, cf premier parametre) */
int SCE_CGetTextureValidSize (int, int);

/* redimensionne une image en tenant compte des limites du materiel */
void SCE_CResizeTextureImage (SCE_CImage*, int, int, int);
void SCE_CResizeTextureImage_ (int, int, int);

/* ajoute une image la liste des images de la texture */
int SCE_CAddTextureImage (SCE_CTexture*, int, SCE_CImage*, int);
int SCE_CAddTextureImage_ (int, SCE_CImage*, int);

/* ajoute une donnee de texture */
int SCE_CAddTextureTexData (SCE_CTexture*, int, SCE_CTexData*, int);
int SCE_CAddTextureTexData_ (int, SCE_CTexData*, int);
/* ajoute une donnee de texture en la dupliquant */
int SCE_CAddTextureTexDataDup (SCE_CTexture*, int, SCE_CTexData*);
int SCE_CAddTextureTexDataDup_ (int, SCE_CTexData*);

/* enleve une image */
SCE_CImage* SCE_CRemoveTextureImage (SCE_CTexture*, int, int);
SCE_CImage* SCE_CRemoveTextureImage_ (int, int);
/* supprime une image */
void SCE_CEraseTextureImage (SCE_CTexture*, int, int);
void SCE_CEraseTextureImage_ (int, int);

/* enleve une donnee de texture */
SCE_CTexData* SCE_CRemoveTextureTexData (SCE_CTexture*, int, int);
SCE_CTexData* SCE_CRemoveTextureTexData_ (int, int);
/* supprime une donnee de texture */
void SCE_CEraseTextureTexData (SCE_CTexture*, int, int);
void SCE_CEraseTextureTexData_ (int, int);

/* charge une texture a partir d'une ou plusieurs images
   (pour cubemap ou specification manuelle des mipmaps) */
SCE_CTexture* SCE_CLoadTexture (int, int, int, int, ...);
SCE_CTexture* SCE_CLoadTextureArg (int, int, int, int, va_list);

/* construit la texture specifiee */
int SCE_CBuildTexture (SCE_CTexture*, int, int);
int SCE_CBuildTexture_ (int, int);

/* met a jour les donnees par defaut de la texture specifiee */
int SCE_CUpdateTexture (SCE_CTexture*, int, int);
int SCE_CUpdateTexture_ (int, int);

/* specifie la texture active et son unite de texture */
void SCE_CUseTexture (SCE_CTexture*, int);

#ifdef __cplusplus
} /* extern "C" */
#endif

#endif /* guard */
